@@ -23,13 +23,13 @@ import trustme
 
 import urllib3.util as util
 import urllib3.util.ssl_
-from dummyserver.server import (
+from dummyserver.socketserver import (
     DEFAULT_CA,
     DEFAULT_CA_KEY,
     DEFAULT_CERTS,
     encrypt_key_pem,
 )
-from dummyserver.testcase import HTTPSDummyServerTestCase
+from dummyserver.testcase import HTTPSHypercornDummyServerTestCase
 from urllib3 import HTTPSConnectionPool
 from urllib3.connection import RECENT_DATE, HTTPSConnection, VerifiedHTTPSConnection
 from urllib3.exceptions import (
@@ -65,7 +65,7 @@ PASSWORD_CLIENT_KEYFILE = "client_password.key"
 CLIENT_CERT = CLIENT_INTERMEDIATE_PEM
 
 
-class TestHTTPS(HTTPSDummyServerTestCase):
+class TestHTTPS(HTTPSHypercornDummyServerTestCase):
     tls_protocol_name: str | None = None
 
     def tls_protocol_not_default(self) -> bool:
@@ -129,7 +129,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
 
         shutil.rmtree(cls.certs_dir)
 
-    def test_simple(self) -> None:
+    def test_simple(self, http_version: str) -> None:
         with HTTPSConnectionPool(
             self.host,
             self.port,
@@ -138,6 +138,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         ) as https_pool:
             r = https_pool.request("GET", "/")
             assert r.status == 200, r.data
+            assert r.headers["server"] == f"hypercorn-{http_version}"
+            assert r.data == b"Dummy server!"
 
     @resolvesLocalhostFQDN()
     def test_dotted_fqdn(self) -> None:
@@ -223,8 +225,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_certs=DEFAULT_CA,
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with warnings.catch_warnings(record=True) as w:
                 r = https_pool.request("GET", "/")
@@ -238,8 +240,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         )
         ctx.load_verify_locations(cafile=DEFAULT_CA)
         with HTTPSConnectionPool(self.host, self.port, ssl_context=ctx) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with mock.patch("warnings.warn") as warn:
                 r = https_pool.request("GET", "/")
@@ -253,8 +255,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA, ssl_context=ctx
         ) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with mock.patch("warnings.warn") as warn:
                 r = https_pool.request("GET", "/")
@@ -274,8 +276,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_cert_dir=str(tmp_path),
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._new_conn()
-            assert conn.__class__ == VerifiedHTTPSConnection
+            with contextlib.closing(https_pool._new_conn()) as conn:
+                assert conn.__class__ == VerifiedHTTPSConnection
 
             with warnings.catch_warnings(record=True) as w:
                 r = https_pool.request("GET", "/")
@@ -323,14 +325,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_certs=self.bad_ca_path,
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with pytest.raises(ssl.SSLError):
                     conn.connect()
 
                 assert conn.sock is not None  # type: ignore[attr-defined]
-            finally:
-                conn.close()
 
     def test_verified_without_ca_certs(self) -> None:
         # default is cert_reqs=None which is ssl.CERT_NONE
@@ -447,6 +446,8 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             # the python ssl module).
             if hasattr(conn.sock, "server_hostname"):  # type: ignore[attr-defined]
                 assert conn.sock.server_hostname == "localhost"  # type: ignore[attr-defined]
+            conn.getresponse().close()
+            conn.close()
 
     def test_assert_fingerprint_md5(self) -> None:
         with HTTPSConnectionPool(
@@ -612,8 +613,7 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             cert_reqs="CERT_NONE",
             ssl_minimum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._new_conn()
-            try:
+            with contextlib.closing(https_pool._new_conn()) as conn:
                 conn.set_tunnel(self.host, self.port)
                 with mock.patch.object(
                     conn, "_tunnel", create=True, return_value=None
@@ -621,8 +621,6 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                     with pytest.warns(InsecureRequestWarning):
                         https_pool._make_request(conn, "GET", "/")
                 conn_tunnel.assert_called_once_with()
-            finally:
-                conn.close()
 
     @requires_network()
     def test_enhanced_timeout(self) -> None:
@@ -633,14 +631,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             retries=False,
             cert_reqs="CERT_REQUIRED",
         ) as https_pool:
-            conn = https_pool._new_conn()
-            try:
+            with contextlib.closing(https_pool._new_conn()) as conn:
                 with pytest.raises(ConnectTimeoutError):
                     https_pool.request("GET", "/")
                 with pytest.raises(ConnectTimeoutError):
                     https_pool._make_request(conn, "GET", "/")
-            finally:
-                conn.close()
 
         with HTTPSConnectionPool(
             TARPIT_HOST,
@@ -659,14 +654,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             retries=False,
             cert_reqs="CERT_REQUIRED",
         ) as https_pool:
-            conn = https_pool._new_conn()
-            try:
+            with contextlib.closing(https_pool._new_conn()) as conn:
                 with pytest.raises(ConnectTimeoutError):
                     https_pool.request(
                         "GET", "/", timeout=Timeout(total=None, connect=SHORT_TIMEOUT)
                     )
-            finally:
-                conn.close()
 
     def test_enhanced_ssl_connection(self) -> None:
         fingerprint = "72:8B:55:4C:9A:FC:1E:88:A1:1C:AD:1B:B2:E7:CC:3E:DB:C8:F9:8A"
@@ -794,15 +786,13 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             self.port,
             ca_certs=DEFAULT_CA,
             ssl_minimum_version=self.tls_version(),
+            ssl_maximum_version=self.tls_version(),
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 conn.connect()
                 if not hasattr(conn.sock, "version"):  # type: ignore[attr-defined]
                     pytest.skip("SSLSocket.version() not available")
                 assert conn.sock.version() == self.tls_protocol_name  # type: ignore[attr-defined]
-            finally:
-                conn.close()
 
     def test_ssl_version_is_deprecated(self) -> None:
         if self.tls_protocol_name is None:
@@ -811,12 +801,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA, ssl_version=self.ssl_version()
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with pytest.warns(DeprecationWarning) as w:
                     conn.connect()
-            finally:
-                conn.close()
 
         assert len(w) >= 1
         assert any(x.category == DeprecationWarning for x in w)
@@ -845,12 +832,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
         with HTTPSConnectionPool(
             self.host, self.port, ca_certs=DEFAULT_CA, ssl_version=ssl_version
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with warnings.catch_warnings(record=True) as w:
                     conn.connect()
-            finally:
-                conn.close()
 
         assert [str(wm) for wm in w if wm.category != ResourceWarning] == []
 
@@ -866,12 +850,9 @@ class TestHTTPS(HTTPSDummyServerTestCase):
             ca_certs=DEFAULT_CA,
             ssl_context=ctx,
         ) as https_pool:
-            conn = https_pool._get_conn()
-            try:
+            with contextlib.closing(https_pool._get_conn()) as conn:
                 with warnings.catch_warnings(record=True) as w:
                     conn.connect()
-            finally:
-                conn.close()
 
         assert [str(wm) for wm in w if wm.category != ResourceWarning] == []
 
@@ -898,7 +879,11 @@ class TestHTTPS(HTTPSDummyServerTestCase):
                 conn = https_pool._get_conn()
                 try:
                     conn.connect()
-                    assert conn.sock.version() == self.tls_protocol_name  # type: ignore[attr-defined]
+                    if maximum_version == TLSVersion.MAXIMUM_SUPPORTED:
+                        # A higher protocol than tls_protocol_name could be negotiated
+                        assert conn.sock.version() >= self.tls_protocol_name  # type: ignore[attr-defined]
+                    else:
+                        assert conn.sock.version() == self.tls_protocol_name  # type: ignore[attr-defined]
                 finally:
                     conn.close()
 
@@ -1147,7 +1132,7 @@ class TestHTTPS_IPV4SAN:
 class TestHTTPS_IPV6SAN:
     @pytest.mark.parametrize("host", ["::1", "[::1]"])
     def test_can_validate_ipv6_san(
-        self, ipv6_san_server: ServerConfig, host: str
+        self, ipv6_san_server: ServerConfig, host: str, http_version: str
     ) -> None:
         """Ensure that urllib3 can validate SANs with IPv6 addresses in them."""
         with HTTPSConnectionPool(
@@ -1158,3 +1143,4 @@ class TestHTTPS_IPV6SAN:
         ) as https_pool:
             r = https_pool.request("GET", "/")
             assert r.status == 200
+            assert r.headers["server"] == f"hypercorn-{http_version}"
